@@ -52,38 +52,18 @@
 char* program_name = 0;
 struct options o;
 
-void src_error(char* title, char* text, char* o) {
-  fprintf(stderr, "%s: error [%s]: %s %s\n", program_name ? program_name : "sourceinstall", title, text, o ? o : "");
-  exit(1);
-}
-
-void src_warning(char* tit, char* txt, char* o) {
-  fprintf(stderr, "%s: warning [%s]: %s %s\n", program_name ? program_name : "sourceinstall", tit, txt, o ? o : "");
-}
-
-void src_info(char* title, char* text, char* opt) {
-  fprintf(stdout, "%s: %s %s\n", title, text, opt ? opt : "");
-}
-
-void src_message(char* title, char* text, char* opt) {
-  if (!o.quiet) {
-    fprintf(stdout, "%s: [%s]: %s %s\n", program_name ? program_name : "sourceinstall", title, text, opt ? opt : "");
-  }
-}
-
 /* CONFIGUREQ param determines if configuration phase is necessary.
    It can assume values 0 = no, 1 = yes. */
 
 int add_package(char* packagename, char* source) {
   SRCINST_PHANDLE h; int rv;
-  char* title = "add_package";
   
   if (!(h = srcinst_new_package(packagename))) {
-    src_warning(title, "package already exists", 0);
+    srcinst_warning(SRCINST_ERR_BUSY, "package already exists", packagename);
     return 0;
   }
 
-  rv = install_package_aux(h, source, 1, 1, title);
+  rv = install_package_aux(h, source, 1, 1);
   return (srcinst_close_package(h) && rv);
 }
 
@@ -92,12 +72,9 @@ int list_packages(char* pattern) {
   struct srcinst_string_list list;
   SRCINST_REGEX reg;
   
-  char* title = "list_packages";
-
   srcinst_list_packages(&list);
 
   if (list.count == 0) {
-    src_warning(title, "no packages", 0);
     return 1;
   }
 
@@ -121,16 +98,16 @@ int list_packages(char* pattern) {
 }
 
 int check_package(char* packagename) {
-  char* tmp; char* title = "check_package";
+  char* tmp; 
   SRCINST_PHANDLE h; 
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_INVALID, "must specify a PACKAGE_NAME", 0);
     return 0;
   }
   
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
@@ -155,7 +132,8 @@ int check_package(char* packagename) {
     
     if ((ssize = srcinst_get_source_size(h)) != (off_t)-1) {
       char* offstr;
-      offstr = srcinst_offtostr(ssize / 1024);
+      offstr = srcinst_offtostr(ssize);
+
       printf("compressed source size (KB): %s\n", offstr);
       free(offstr);
     }
@@ -172,7 +150,7 @@ int check_package(char* packagename) {
     struct srcinst_string_list* list;
     if ((ssize = srcinst_get_installed_size(h)) != (off_t)-1) {
       char* offstr;
-      offstr = srcinst_offtostr(ssize / 1024);
+      offstr = srcinst_offtostr(ssize);
       printf("installed size (KB): %s\n", offstr);
       free(offstr);
     }
@@ -186,109 +164,123 @@ int check_package(char* packagename) {
     if (list->count > 0) {
       int i; int install_check; /* 1 = ok, 0 = error */
       install_check = 1;
-      printf("\ninstalled files and directories:\n");
 
       for (i = 0; i < list->count; i++) {
 	int missing;
+
 	missing = srcinst_file_ltype(list->strings[i]) == SRCINST_TYPE_NONE;
-	if (install_check && missing)
+	if (missing) {
+	  printf("%s [%s]\n", list->strings[i], "MISSING");
 	  install_check = 0;
-	printf("%s [%s]\n", list->strings[i], missing ? "MISSING" : "ok");
+	}
       }
 
-      if (!install_check) {
-	src_warning(title, "some files seem unreachable (MISSING)", 0);
-      }
+      if (install_check)
+	printf("installed files: all present and reachable.\n");
+
     } else {
-      src_warning(title, "no files installed, or no info available", 0);
+      printf("installed files: no files installed, or no info available.\n");
     }
   }
 
-  return (srcinst_close_package(h));
+  return srcinst_close_package(h);
 }
 
-int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int check_conflicts, char* title) {
+int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int check_conflicts) {
 
-  int staged_install;
+  SRCINST_ERR err;
   SRCINST_MFEATS mf; int rv = 0;
   
-  if (srcinst_init_build(h, source, o.subdir) != SRCINST_ERR_OK) {
-    src_warning(title, "could not initialize build", 0);
+  if ((err = srcinst_init_build(h, source, o.subdir)) != SRCINST_ERR_OK) {
+    srcinst_warning(err, "could not initialize build", 0);
     return 0;
   }
 
   if (configureq) {
-    if (srcinst_configure(h, o.prefix, &(o.configure_strings)) != SRCINST_ERR_OK) {
-      src_warning(title, "configuration failed", 0);
+    if ((err = srcinst_configure(h, o.prefix, &(o.configure_strings)))
+	!= SRCINST_ERR_OK) {
+      srcinst_warning(err, "configuration failed", 0);
       if (!o.force)
 	return 0;
     }
   }
   
   if (!(mf = srcinst_detect_makefile_features(h))) {
-    src_warning(title, "failed to detect makefile features", 0);
-    goto end_proc;
+    srcinst_warning(SRCINST_ERR_ACTION,
+		    "failed to detect makefile features", 0);
+    return 0;
   }
-  
-  staged_install = srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) ||
-    srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT);
- 
-  if (!staged_install) {
-    src_warning(title, "build system does not support staged installs", 0);
-    if (!o.force)
+
+  if (!srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) &&
+      !srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT)) {
+
+    if (!srcinst_is_feature_supported(mf, SRCINST_FEATURE_PREFIX)) {
+      srcinst_warning(SRCINST_ERR_POLICY,
+		      "build system does not support staged installs", 0);
       goto end_proc;
+
+    } else if (o.force) {
+      srcinst_warning(SRCINST_ERR_POLICY,
+		      "build system does not support staged installs, "
+		      "forcing prefix redirection", 0);
+    } else {
+      srcinst_warning(SRCINST_ERR_POLICY,
+		      "build system does not support staged installs. Use "
+		      "--force to use prefix redirection (discouraged)", 0);
+      goto end_proc;
+    }
   }
   
   if (o.source) {
-    if (srcinst_install_source(h, mf, o.src_compress) != SRCINST_ERR_OK) {
-      src_warning(title, "source installation failed", 0);
+    if ((err = srcinst_install_source(h, mf, o.src_compress))
+	!= SRCINST_ERR_OK) {
+      srcinst_warning(err, "source installation failed", 0);
       goto end_proc;
     }
   }
 
-  if (srcinst_build(h, mf) != SRCINST_ERR_OK) {
-    src_warning(title, "compilation phase failed", 0);
+  if ((err = srcinst_build(h, mf)) != SRCINST_ERR_OK) {
+    srcinst_warning(err, "compilation phase failed", 0);
     goto end_proc;
   }
 
   if (o.install) {
-    if (staged_install) {
-      SRCINST_ERR tst_err;
-      if ((tst_err = srcinst_test_install(h, mf, o.strip)) != SRCINST_ERR_OK) {
-	src_warning(title, "test installation reports a fatal error", 0);
-	goto end_proc;
-      }
 
-      if (check_conflicts) {
-	struct srcinst_string_list conflicts;
-
-	if (!srcinst_get_conflict_list(h, &conflicts)) {
-	  src_warning(title, "failed to perform conflict test", 0);
-	  goto end_proc;
-	}
-
-	if (conflicts.count > 0) {
-	  int i;
-	  src_warning(title, "installation would overwrite files claimed by other packages", 0);
-	  for (i = 0; i < conflicts.count; i++) {
-	    src_warning("already claimed", conflicts.strings[i], 0);
-	  }
-	  
-	  if (!o.force) {
-	    srcinst_free_string_list(&conflicts);
-	    goto end_proc;
-	  }
-	}
-	srcinst_free_string_list(&conflicts);
-      }
+    if ((err = srcinst_test_install(h, mf, o.strip)) != SRCINST_ERR_OK) {
+      srcinst_warning(err, "test installation failed", 0);
+      goto end_proc;
     }
 
-    if (srcinst_install_binary(h, mf, o.strip) != SRCINST_ERR_OK) {
-      src_warning(title, "binary installation failed", 0);
+    if (check_conflicts) {
+      struct srcinst_string_list conflicts;
+      
+      if (!srcinst_get_conflict_list(h, &conflicts)) {
+	srcinst_warning(SRCINST_ERR_ACTION,
+			"conflict test could not be performed", 0);
+	goto end_proc;
+      }
+      
+      if (conflicts.count > 0) {
+	int i;
+	srcinst_warning(SRCINST_ERR_BUSY, "installation would overwrite "
+			"files claimed by other packages:", 0);
+	for (i = 0; i < conflicts.count; i++) {
+	  fprintf(stderr, "%s\n", conflicts.strings[i]);
+	}
+	
+	srcinst_free_string_list(&conflicts);
+	
+	if (!o.force)
+	  goto end_proc;
+      }
+    }
+    
+    if ((err = srcinst_install_binary(h, mf, o.strip)) != SRCINST_ERR_OK) {
+      srcinst_warning(err, "binary installation failed", 0);
       goto end_proc;
     }
   }
-
+  
   rv = 1;
   
  end_proc:
@@ -299,15 +291,14 @@ int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int che
 int install_package(char* packagename) {
   int rv; SRCINST_PHANDLE h; char* source; char* prefix;
   int configureq;
-  char* title = "install_package";
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
 
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
@@ -317,231 +308,204 @@ int install_package(char* packagename) {
     (prefix && strcmp(o.prefix, prefix) != 0);
 
   if (!(source = srcinst_get_source_location(h))) {
-    src_warning(title, "source not available for", packagename);
+    srcinst_warning(SRCINST_ERR_MISSING,
+		    "source code not available for", packagename);
     rv = 0;
   } else {
-    rv = install_package_aux(h, source, configureq, 1, title);
+    rv = install_package_aux(h, source, configureq, 1);
   }
 
   return (srcinst_close_package(h) && rv);
 }
 
 int uninstall_binary(char* packagename) {
-  int rv; SRCINST_PHANDLE h;
-  char* title = "uninstall_binary";
+  int rv = 0; SRCINST_ERR err; SRCINST_PHANDLE h;
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
 
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
   if (!srcinst_is_installed(h)) {
-    src_warning(title, "not installed:", packagename);
-    rv = 0;
+    srcinst_warning(SRCINST_ERR_INVALID, "not an installed package",
+		    packagename);
 
-  } else if (!(rv = srcinst_uninstall_binary(h) == SRCINST_ERR_OK)) {
-    src_warning(title, "binary uninstallation failed", 0);
+  } else if (!(rv = ((err = srcinst_uninstall_binary(h)) == SRCINST_ERR_OK))) {
+    srcinst_warning(err, "binary uninstallation failed", 0);
   }
 
-  return srcinst_close_package(h) && rv;
+  return (srcinst_close_package(h) && rv);
 }
 
 int uninstall_source(char* packagename) {
-  int rv; SRCINST_PHANDLE h;
-  char* title = "uninstall_source";
+  int rv = 0; SRCINST_ERR err; SRCINST_PHANDLE h;
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
   
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
   
   if (!srcinst_is_source_available(h)) {
-    src_warning(title, "source not available:", packagename);
-    rv = 0;
+    srcinst_warning(SRCINST_ERR_MISSING, "source code not available", 0);
     
-  } else if (!(rv = srcinst_uninstall_source(h) == SRCINST_ERR_OK)) {
-    src_warning(title, "source uninstallation failed", 0);
+  } else if (!(rv = ((err = srcinst_uninstall_source(h)) == SRCINST_ERR_OK))) {
+    srcinst_warning(err, "source uninstallation failed", 0);
   }
   
-  return srcinst_close_package(h) && rv;
+  return (srcinst_close_package(h) && rv);
 }
 
-int upgrade_package(char* packagename, char* oldname, char* src) {
-  SRCINST_PHANDLE h, oldh; int result;
-  char* title = "upgrade_package";
-  char* descr, *descr_l;
+int upgrade_package(char* newname, char* oldname, char* src) {
+  SRCINST_PHANDLE h; int rv;
   
-  h = oldh = 0; result = 0;
+  h = 0; rv = 0;
 
-  if (!(oldh = srcinst_open_package(oldname))) {
-    src_warning(title, "could not find package", oldname);
+  if (!(h = srcinst_upgrade_package(oldname, newname))) {
+    srcinst_warning(SRCINST_ERR_ACTION, "cannot start upgrade, "
+		    "check package names", 0);
     goto end_proc;
   }
 
-  if ((srcinst_get_installed_files(oldh))->count == 0) {
-    src_warning(title, "no information about installed files available for package", oldname);
+  if (!install_package_aux(h, src, 1, 0)) {
+    srcinst_warning(SRCINST_ERR_ACTION, 
+		    "could not install new package version", 0);
     goto end_proc;
   }
 
-  if (!(h = srcinst_new_package(packagename))) {
-    src_warning(title, "package already exists", 0);
-    goto end_proc;
-  }
-
-  if (!install_package_aux(h, src, 1, 0, title)) {
-    src_warning(title, "could not install new package version", 0);
-    goto end_proc;
-  }
-
-  if ((descr = srcinst_get_descr(oldh))) {
-    srcinst_set_descr(h, descr);
-  }
-  
-  if ((descr_l = srcinst_get_descr_long(oldh))) {
-    srcinst_set_descr_long(h, descr_l);
-  }
-
-  result = 1;
+  rv = 1;
 
  end_proc:
-  
-  if (result) {
-    result = srcinst_upgrade_package(oldh, h) == SRCINST_ERR_OK;
-  }
-
-  return result;
+  return (srcinst_close_package(h) && rv);
 }
 
 int remove_package(char* packagename) {
-  int rv; SRCINST_PHANDLE h;
-  char* title = "remove_package";
+  int rv = 0; SRCINST_ERR err; SRCINST_PHANDLE h;
   
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
   
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
   if (srcinst_is_installed(h)) {
-    if (!(rv = srcinst_uninstall_binary(h) == SRCINST_ERR_OK)) {
-      src_warning(title, "binary uninstallation failed", 0);
+    if (!(rv = ((err = srcinst_uninstall_binary(h)) == SRCINST_ERR_OK))) {
+      srcinst_warning(err, "binary uninstallation failed", 0);
       goto end_proc;
     }
   }
 
   if (srcinst_is_source_available(h)) {
-    if (!(rv = srcinst_uninstall_source(h) == SRCINST_ERR_OK)) {
-      src_warning(title, "source uninstallation failed", 0);
+    if (!(rv = ((err = srcinst_uninstall_source(h)) == SRCINST_ERR_OK))) {
+      srcinst_warning(err, "source uninstallation failed", 0);
     }
   }
-
+  
  end_proc:
-  return srcinst_close_package(h) && rv;
+  return (srcinst_close_package(h) && rv);
 }
 
 int export(char* packagename, char* export_format, char* filename) {
-  SRCINST_PHANDLE h; int rv; SRCINST_EXPORT fmt;
-  char* title = "export";
+  SRCINST_PHANDLE h; SRCINST_EXPORT fmt;
+  SRCINST_ERR err; int rv = 0; 
 
   fmt = srcinst_str_to_export_format(export_format);
   if (fmt == SRCINST_EXPORT_UNKNOWN) {
-    src_warning(title, "invalid export format:", export_format);
+    srcinst_warning(SRCINST_ERR_INVALID, "export format", export_format);
     return 0;
   }
 
   if (!packagename) {
     if (fmt == SRCINST_EXPORT_LST) {
-      src_warning(title, "must specify a PACKAGE_NAME for LST format", 0);
+      srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
       return 0;
     }
     h = (SRCINST_PHANDLE)0;
 
   } else if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
   if (fmt == SRCINST_EXPORT_LST && 
       (!srcinst_is_installed(h) || !srcinst_get_installed_files(h)->count)) { 
-    src_warning(title, "no installed files information available on", packagename);
+    srcinst_warning(SRCINST_ERR_INVALID, "no installed files information available for", packagename);
     rv = 0;
 
-  } else if (!(rv = srcinst_export_info(h, filename, fmt) == SRCINST_ERR_OK)) {
-    src_warning(title, "failed to export package information", 0);
+  } else if (!(rv = ((err = srcinst_export_info(h, filename, fmt))
+		     == SRCINST_ERR_OK))) {
+    srcinst_warning(err, "failed to export package information", 0);
   }
 
   if (h)
-    rv = srcinst_close_package(h) && rv;
+    rv = (srcinst_close_package(h) && rv);
 
   return rv;
 }
 
 int export_binary(char* packagename, char* filename) {
-  SRCINST_PHANDLE h; SRCINST_ERR result;
-  char* title = "export_binary";
+  SRCINST_PHANDLE h; SRCINST_ERR err;
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
 
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
-  if ((result = srcinst_export_binary(h, filename, o.src_compress)) != SRCINST_ERR_OK) {
-    src_warning(title, "failed to export binary package", packagename);
+  if ((err = srcinst_export_binary(h, filename, o.src_compress))
+      != SRCINST_ERR_OK) {
+    srcinst_warning(err, "failed to export binary package", packagename);
   }
-
-  return (srcinst_close_package(h) && result == SRCINST_ERR_OK);
+  
+  return (srcinst_close_package(h) && err == SRCINST_ERR_OK);
 }
 
 int detect_builddirs(char* packagename, char* source) {
-  SRCINST_PHANDLE h; 
+  SRCINST_PHANDLE h; SRCINST_ERR err;
   struct srcinst_string_list scripts;
-  char* title = "detect_builddirs";
 
   if (packagename) {
-    src_warning(title, "ignoring package name", packagename);
+    srcinst_warning(SRCINST_ERR_INVALID, "ignoring package name", packagename);
   }
 
   if (!(h = srcinst_new_package(0))) {
-    src_warning(title, "could not create temporary package", 0);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not create temp package", 0);
     return 0;
   }
 
   if (o.subdir) {
-    src_warning(title, "ignoring -D, --subdir option", 0);
+    srcinst_warning(SRCINST_ERR_INVALID, "ignoring -D, --subdir option", 0);
   }
-
-  if (srcinst_init_build(h, source, 0) != SRCINST_ERR_OK ||
-      srcinst_find_configure(h, &scripts) != SRCINST_ERR_OK) {
-    src_warning(title, "could not scan temporary build", 0);
+  
+  if ((err = srcinst_init_build(h, source, 0)) != SRCINST_ERR_OK ||
+      (err = srcinst_find_configure(h, &scripts)) != SRCINST_ERR_OK) {
+    srcinst_warning(err, "could not scan temporary build", 0);
     srcinst_close_package(h);
     return 0;
   }
 
   /* first: check configure script */
 
-  printf("configure director%s", scripts.count > 1 ? "ies:\n" : "y: ");
-
   if (scripts.count == 0) {
-    printf("NONE\nNo configure script found, and no configure-generation script detected.\nIt is recommended _not_ to install this package.\n");
+    /* no output. The package does not provide a configure script,
+       or a configure-generating script */
 
   } else {
     int i;
@@ -550,7 +514,7 @@ int detect_builddirs(char* packagename, char* source) {
       chdir(srcinst_get_dir(SRCINST_DIR_BUILD));
       chdir(srcinst_get_packdir(h));
       dname = srcinst_dirname(scripts.strings[i]);
-      printf("%s ", dname);
+      printf("%s\t", dname);
 
       if (chdir(dname) == 0 && srcinst_detect_autoconf(h, &version) ==
 	  SRCINST_ERR_OK) {
@@ -572,19 +536,19 @@ int detect_builddirs(char* packagename, char* source) {
 int detect_feats(char* packagename, char* source) {
   SRCINST_PHANDLE h; float version; SRCINST_MFEATS mf;
   char configure_detected, configure_test;
-  char* title = "detect_feats";
+  SRCINST_ERR err;
 
   if (packagename) {
-    src_warning(title, "ignoring package name", packagename);
+    srcinst_warning(SRCINST_ERR_INVALID, "ignoring package name", packagename);
   }
 
   if (!(h = srcinst_new_package(0))) {
-    src_warning(title, "could not create temporary package", 0);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not create temp package", 0);
     return 0;
   }
 
-  if (srcinst_init_build(h, source, o.subdir) != SRCINST_ERR_OK) {
-    src_warning(title, "could not initialize temporary build", 0);
+  if ((err = srcinst_init_build(h, source, o.subdir)) != SRCINST_ERR_OK) {
+    srcinst_warning(err, "could not initialize temporary build", 0);
     srcinst_close_package(h);
     return 0;
   }
@@ -597,11 +561,10 @@ int detect_feats(char* packagename, char* source) {
 
   mf = srcinst_detect_makefile_features(h);
 
-  printf("configure script: ");
+  printf("configure: ");
   if (!configure_detected) {
-    printf("NOT DETECTED\n");
+    printf("no\n");
   } else {
-    printf("yes, ");
     if (version < 0.1)
       printf("custom\n");
     else
@@ -610,23 +573,21 @@ int detect_feats(char* packagename, char* source) {
     printf("configure test: %s", configure_test ? "success" : "failure");
   }
 
-  printf("\nmakefile: %s\n", mf ? "detected" : "NOT DETECTED");
+  printf("\nmakefile: %s\n", mf ? "yes" : "no");
 
   if (mf) {
-    printf("  DESTDIR: %s\n",
+    printf("DESTDIR: %s\n",
 	   srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) ?
-	   "supported" :
+	   "yes" : "no");
+    printf("INSTALL_ROOT: %s\n",
 	   srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT) ?
-	   "using old replacement INSTALL_ROOT" :
-	   "NOT SUPPORTED");
-
-    printf("  uninstall: %s\n",
+	   "yes" : "no");
+    printf("uninstall: %s\n",
 	   srcinst_is_feature_supported(mf, SRCINST_FEATURE_UNINSTALL) ?
-	   "supported" : "not supported");
-
-    printf("  install-strip: %s\n",
+	   "yes" : "no");
+    printf("install-strip: %s\n",
 	   srcinst_is_feature_supported(mf, SRCINST_FEATURE_STRIP) ?
-	   "supported" : "not supported");
+	   "yes" : "no");
   }
 
   if (mf)
@@ -637,66 +598,65 @@ int detect_feats(char* packagename, char* source) {
 }
 
 int detect_opts(char* packagename, char* source) {
-  SRCINST_PHANDLE h;
+  SRCINST_PHANDLE h; SRCINST_ERR err;
   struct srcinst_configure_sets option_sets; char options_detected;
-  char* title = "detect_feats";
 
   if (packagename) {
-    src_warning(title, "ignoring package name", packagename);
+    srcinst_warning(SRCINST_ERR_INVALID, "ignoring package name", packagename);
   }
 
   if (!(h = srcinst_new_package(0))) {
-    src_warning(title, "could not create temporary package", 0);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not create temp package", 0);
     return 0;
   }
 
-  if (srcinst_init_build(h, source, o.subdir) != SRCINST_ERR_OK) {
-    src_warning(title, "could not initialize temporary build", 0);
+  if ((err = srcinst_init_build(h, source, o.subdir)) != SRCINST_ERR_OK) {
+    srcinst_warning(err, "could not initialize temporary build", 0);
     srcinst_close_package(h);
     return 0;
   }
 
-  options_detected = srcinst_detect_configure_opt(h, &option_sets) == SRCINST_ERR_OK;
+  options_detected =
+    (err = srcinst_detect_configure_opt(h, &option_sets)) == SRCINST_ERR_OK;
 
   if (!options_detected) {
-    printf("could not detect configure options.\n");
+    srcinst_warning(err, "could not detect configure options", 0);
+    
   } else {
-    printf("[default --prefix option assumed to be recognized]\n");
     if (option_sets.packs.count > 0) {
-      printf("Option class detected: optional packages\n");
+      printf("Option class: Optional Packages\n");
       print_option_set(&option_sets.packs);
     }
     if (option_sets.feats.count > 0) {
-      printf("Option class detected: optional features\n");
+      printf("Option class: Optional Features\n");
       print_option_set(&option_sets.feats);
     }
     if (option_sets.envs.count > 0) {
-      printf("Option class detected: influential environment variables\n");
+      printf("Option class: Influential Environment Variables\n");
       print_option_set(&option_sets.envs);
     }
     if (option_sets.olds.count > 0) {
-      printf("Option class detected: old (2.13 style) features/packages\n");
+      printf("Option class: Old-style (2.13) features/packages\n");
       print_option_set(&option_sets.olds);
     }
 
     srcinst_free_configure_opt(&option_sets);
   }
-
+  
   srcinst_close_package(h);
   return 1;
 }
 
 int describe_package(char* packagename, char* string) {
   SRCINST_PHANDLE h; int rv;
-  char* title = "describe_package";
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "PACKAGE_NAME", 0);
     return 0;
   }
 
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
 
@@ -709,37 +669,37 @@ int describe_package(char* packagename, char* string) {
     if ((mark = strchr(string, '\n'))) {
       *mark = 0;
       rv = srcinst_set_descr(h, string) && srcinst_set_descr_long(h, mark + 1);
-
+      
     } else {
       rv = srcinst_set_descr(h, string);
     }
   }
 
-  return srcinst_close_package(h) && rv;
+  return (srcinst_close_package(h) && rv);
 }
 
 int rename_package(char* packagename, char* newname) {
-  SRCINST_PHANDLE h; int rv; char* title = "rename_package";
+  SRCINST_PHANDLE h; int rv; 
 
   if (!packagename) {
-    src_warning(title, "must specify a PACKAGE_NAME", 0);
+    srcinst_warning(SRCINST_ERR_MISSING, "must specify a PACKAGE_NAME", 0);
     return 0;
   }
 
   if (!(h = srcinst_open_package(packagename))) {
-    src_warning(title, "could not open package", packagename);
+    srcinst_warning(SRCINST_ERR_ACTION, "could not open package", packagename);
     return 0;
   }
   
   rv = srcinst_rename_package(h, newname) == SRCINST_ERR_OK;
-  return srcinst_close_package(h) && rv;
+  return (srcinst_close_package(h) && rv);
 }
 
 int parse_cmdline(int argc, char* argv[]) {
-  int i, j, k, selected_action; char* title; struct option_data* o_set[] = {
+  int i, j, k, selected_action;
+  struct option_data* o_set[] = {
     action_list, option_list, 0
   };
-  title = "parse_cmdline";
 
   o.package = 0;		/* no package name selected */
   selected_action = -1;
@@ -753,7 +713,8 @@ int parse_cmdline(int argc, char* argv[]) {
 
     if (argv[i][0] != '-') {
       if (o.package) {
-	src_error(title, "package already specified:", argv[i]);
+	srcinst_error(SRCINST_ERR_BUSY, "package name already specified",
+		      argv[i]);
       }
       o.package = srcinst_strdup(argv[i]);
       continue;
@@ -762,15 +723,15 @@ int parse_cmdline(int argc, char* argv[]) {
     for (j = 0; o_set[j]; j++) {
       for (k = 0; o_set[j][k].short_option; k++) {
 	struct option_data* data = o_set[j] + k;
-	if (argv[i][1] == data->short_option ||
+	if ((argv[i][1] == data->short_option && argv[i][2] == '\0') ||
 	    (argv[i][1] == '-' && 
 	     strcmp(argv[i] + 2, data->long_option) == 0)) {
 	  found = 1;
 	  
 	  if (o_set[j] == action_list) {
 	    if (selected_action != -1) {
-	      src_error(title, "multiple actions requested:",
-			data->long_option);
+	      srcinst_error(SRCINST_ERR_BUSY, "multiple actions requested",
+			    data->long_option);
 	    }
 	    selected_action = k;
 	  }
@@ -782,8 +743,10 @@ int parse_cmdline(int argc, char* argv[]) {
 	      int x;
 	      for (x = 0; data->params[x]; x++) {
 		i++;
-		if (i >= argc || argv[i][0] == '-' && argv[i][1] != '\0') {
-		  src_error(title, "missing required option parameter:", data->long_option);
+		if (i >= argc || (argv[i][0] == '-' && argv[i][1] != '\0')) {
+		  srcinst_error(SRCINST_ERR_MISSING, 
+				"required option parameter",
+				data->long_option);
 		}
 		data->values[x] = argv[i];
 	      }
@@ -791,15 +754,18 @@ int parse_cmdline(int argc, char* argv[]) {
 	      break;
 	    }
 	  case 1:
-	    src_error(title, "unique option already defined:", data->long_option);
+	    srcinst_error(SRCINST_ERR_BUSY, "unique option already defined",
+			  data->long_option);
 	    
 	  case 2:
 	    {
 	      if (++i >= argc) {
-		src_error(title, "missing required string for", data->long_option);
+		srcinst_error(SRCINST_ERR_MISSING, "required string for",
+			      data->long_option);
 	      }
 	      
-	      srcinst_add_string_list((struct srcinst_string_list*)(data->values[0]), argv[i]);
+	      srcinst_add_string_list((struct srcinst_string_list*)
+				      (data->values[0]), argv[i]);
 	      break;
 	    }
 	  default:
@@ -812,27 +778,36 @@ int parse_cmdline(int argc, char* argv[]) {
       }
     }
 
-    src_error(title, "unrecognized option:", argv[i]);
+    srcinst_error(SRCINST_ERR_INVALID, "unrecognized option:", argv[i]);
 
   arg_found:
     ;
   }
 
   if (selected_action == -1) {
-    /* ugly hack for partial backward compatibility:
-       default action is Add with a single filename argument */
-    selected_action = 3;
-    action_list[selected_action].values[0] = o.package;
-    o.package = 0;
+    /* no action is selected */
 
-  } else {
-    /* perform quick actions */
+    if (o.package) {
+      /* if non-option argument is specified,
+	 then provide partial backward compatibility:
+	 default action is Add with a single filename argument */
 
-    switch (action_list[selected_action].short_option) {
-    case 'h': cmdline_help();
-    case 'V': cmdline_version();
+      selected_action = 3;	/* warning! add must be at position 3!  */
+      action_list[selected_action].values[0] = o.package;
+      o.package = 0;		/* reset o.package to 'default package name' */
+      return selected_action;
     }
+    
+    return -1;
   }
+
+  /* some action selected */
+
+  switch (action_list[selected_action].short_option) {
+  case 'h': cmdline_help();
+  case 'V': cmdline_version();
+  }
+  
   return selected_action;
 }
 
@@ -906,7 +881,6 @@ void cmdline_version(void) {
 
 int cmdline_options_apply(int selected_action) {
   int action_code; struct option_data* data; int i;
-  char* title = "cmdline_options_apply";
   
   action_code = action_list[selected_action].short_option;
 
@@ -918,7 +892,8 @@ int cmdline_options_apply(int selected_action) {
 	o.source = 0; break;
       case 'z':
 	if ((o.src_compress = srcinst_str_to_compression_format(data->values[0])) == SRCINST_COMP_UNKNOWN)
-	  src_error(title, "invalid compression format:", data->values[0]);
+	  srcinst_error(SRCINST_ERR_INVALID, "compression format",
+			data->values[0]);
 	break;
       case 't':
 	o.strip = 1; break;
@@ -1001,11 +976,10 @@ int cmdline_options_apply(int selected_action) {
 }
 
 int main(int argc, char* argv[]) {
-  char* title = "main";
   int rv, todo;
 
   if (!srcinst_init(SRCINST_FLAG_LOAD_ONDEMAND)) {
-    src_error(title, "could not initialize libsrcinst", 0);
+    srcinst_error(SRCINST_ERR_CORE, "could not initialize libsrcinst", 0);
   }
   atexit(srcinst_fini);
   program_name = (argv[0] ? srcinst_basename(argv[0]) : "sourceinstall");
@@ -1013,11 +987,11 @@ int main(int argc, char* argv[]) {
   init_options(&o);
   
   if ((todo = parse_cmdline(argc, argv)) == -1) {
-    src_error(title, "nothing to do. Try -h or --help", 0);
+    srcinst_error(SRCINST_ERR_ACTION, "nothing to do. Try -h or --help", 0);
   }
-
+  
   if (!(rv = cmdline_options_apply(todo))) {
-    src_error(title, "action failed", 0);
+    srcinst_error(SRCINST_ERR_ACTION, "failure", 0);
   }
 
   return 0;
@@ -1026,13 +1000,13 @@ int main(int argc, char* argv[]) {
 void print_option_set(struct srcinst_configure_options* set) {
   int i;
   for (i = 0; i < set->count; i++) {
-    printf("Option: %s  " , set->options[i].key);
+    printf("Option: %s", set->options[i].key);
     if (set->options[i].param) {
-      printf("Param: %s", set->options[i].param);
+      printf("\t%s", set->options[i].param);
     }
     printf("\n");
     if (set->options[i].comment) {
-      printf("Comment: %s\n", set->options[i].comment);
+      printf("Comment: %s", set->options[i].comment);
     }
     printf("\n");
   }
