@@ -24,14 +24,10 @@
 #ifndef SOURCEINSTALL_H
 #define SOURCEINSTALL_H
 
-void src_error(char* title, char* text, char* opt);
-void src_warning(char* title, char* text, char* opt);
-void src_info(char* title, char* text, char* opt);
-void src_message(char* title, char* text, char* opt);
 int add_package(char* packagename, char* source);
 int check_package(char* packagename);
 int list_packages(char* pattern);
-int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int check_conflicts, char* title);
+int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int check_conflicts);
 int install_package(char* packagename);
 int uninstall_package(char* packagename);
 int upgrade_package(char* packagename, char* oldname, char* source);
